// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO40/legacy/DIO40_Linux_1.00.0_GSC_DN/lib/gpio.c $
// $Rev: 32426 $
// $Date:$

#include "dio40_lib.h"



/******************************************************************************
*
*	Function:	dio40_gpio_a_dir_get
*
*	Purpose:
*
*		Return the direction state of the group A GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		dir		Tell the caller the direction of the pins. 0 = input, 1 = output
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a_dir_get(int fd, u8* dir)
{
	u32	ccar;
	int	err;

	err	= dio40_reg_read(fd, DIO40_GSC_CCAR, &ccar);

	if (dir)
		dir[0]	= (u8) (ccar & 0xFF);

	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a_dir_mod
*
*	Purpose:
*
*		Update the direction state of some of the group A GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		dir		The direction of the pins. 0 = input, 1 = output
*
*		mask	The set of LEDs to access. If a bit is set here, then the LED
*				is accessed. If the bit is not set, then it is unmodified.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a_dir_mod(int fd, u8 dir, u8 mask)
{
	int	err;

	err	= dio40_reg_mod(fd, DIO40_GSC_CCAR, dir, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a_dir_set
*
*	Purpose:
*
*		Update the direction state of the group A GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		dir		The direction of the pins. 0 = input, 1 = output
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a_dir_set(int fd, u8 dir)
{
	int	err;

	err	= dio40_reg_mod(fd, DIO40_GSC_CCAR, dir, 0xFF);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a_in_get
*
*	Purpose:
*
*		Return the raw input of the group A GPIO cable pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		data	Give the caller the cable data.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a_in_get(int fd, u8* data)
{
	u32	ciar;
	int	err;

	err	= dio40_reg_read(fd, DIO40_GSC_CIAR, &ciar);

	if (data)
		data[0]	= (u8) (ciar & 0xFF);

	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a_out_get
*
*	Purpose:
*
*		Return the latched state of the group A GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		data	Give the caller the latched data.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a_out_get(int fd, u8* data)
{
	u32	coar;
	int	err;

	err	= dio40_reg_read(fd, DIO40_GSC_COAR, &coar);

	if (data)
		data[0]	= (u8) (coar & 0xFF);

	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a_out_mod
*
*	Purpose:
*
*		Update the latched state of some of the group A GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		data	The data to apply.
*
*		mask	The set of latches to access. If a bit is set here, then the
*				latch is accessed. If the bit is not set, then it is unmodified.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a_out_mod(int fd, u8 data, u8 mask)
{
	int	err;

	err	= dio40_reg_mod(fd, DIO40_GSC_COAR, data, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a_out_set
*
*	Purpose:
*
*		Update the latched state of the group A GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		data	The data to apply.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a_out_set(int fd, u8 data)
{
	int	err;

	err	= dio40_reg_mod(fd, DIO40_GSC_COAR, data, 0xFF);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a0_tx_clock_get
*
*	Purpose:
*
*		Tell the caller if the A0 Tx Clock option is enabled.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		get		Tell the caller if the clock is enabled.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a0_tx_clock_get(int fd, u8* get)
{
	u32	ccar;
	int	err;

	err	= dio40_reg_read(fd, DIO40_GSC_CCAR, &ccar);

	if (get)
		get[0]	= ((ccar & 0x100) && (ccar & 0x1)) ? 1 : 0;

	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a0_tx_clock_set
*
*	Purpose:
*
*		Enable or disable the A0 Tx Clock option.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		set		Enable or disable the option.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a0_tx_clock_set(int fd, int set)
{
	int	err;
	u32	mask	= 0x101;
	u32	value	= set ? mask : 0;

	err		= dio40_reg_mod(fd, DIO40_GSC_CCAR, value, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a7_tx_clock_get
*
*	Purpose:
*
*		Tell the caller if the A7 Tx Clock option is enabled.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		get		Tell the caller if the clock is enabled.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a7_tx_clock_get(int fd, u8* get)
{
	u32	ccar;
	int	err;

	err	= dio40_reg_read(fd, DIO40_GSC_CCAR, &ccar);

	if (get)
		get[0]	= ((ccar & 0x10000) && (ccar & 0x80)) ? 1 : 0;

	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_a7_tx_clock_set
*
*	Purpose:
*
*		Enable or disable the A7 Tx Clock option.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		set		Enable or disable the option.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_a7_tx_clock_set(int fd, int set)
{
	int	err;
	u32	mask	= 0x10080;
	u32	value	= set ? mask : 0;

	err		= dio40_reg_mod(fd, DIO40_GSC_CCAR, value, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_b_dir_get
*
*	Purpose:
*
*		Return the direction state of the group B GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		dir		Tell the caller the direction of the pins. 0 = input, 1 = output
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_b_dir_get(int fd, u32* dir)
{
	int	err;

	if (dir)
		err	= dio40_reg_read(fd, DIO40_GSC_CCBR, dir);
	else
		err	= 0;

	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_b_dir_mod
*
*	Purpose:
*
*		Update the direction state of some of the group B GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		dir		The direction of the pins. 0 = input, 1 = output
*
*		mask	The set of LEDs to access. If a bit is set here, then the LED
*				is accessed. If the bit is not set, then it is unmodified.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_b_dir_mod(int fd, u32 dir, u32 mask)
{
	int	err;

	err	= dio40_reg_mod(fd, DIO40_GSC_CCBR, dir, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_b_dir_set
*
*	Purpose:
*
*		Update the direction state of the group B GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		dir		The direction of the pins. 0 = input, 1 = output
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_b_dir_set(int fd, u32 dir)
{
	int	err;

	err	= dio40_reg_write(fd, DIO40_GSC_CCBR, dir);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_b_in_get
*
*	Purpose:
*
*		Return the raw input of the group B GPIO cable pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		data	Give the caller the cable data.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_b_in_get(int fd, u32* data)
{
	int	err;

	if (data)
		err	= dio40_reg_read(fd, DIO40_GSC_CIBR, data);
	else
		err	= 0;

	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_b_out_get
*
*	Purpose:
*
*		Return the latched state of the group B GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		data	Give the caller the latched data.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_b_out_get(int fd, u32* data)
{
	int	err;

	if (data)
		err	= dio40_reg_read(fd, DIO40_GSC_COBR, data);
	else
		err	= 0;

	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_b_out_mod
*
*	Purpose:
*
*		Update the latched state of some of the group B GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		data	The data to apply.
*
*		mask	The set of latches to access. If a bit is set here, then the
*				latch is accessed. If the bit is not set, then it is unmodified.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_b_out_mod(int fd, u32 data, u32 mask)
{
	int	err;

	err	= dio40_reg_mod(fd, DIO40_GSC_COBR, data, mask);
	return(err);
}



/******************************************************************************
*
*	Function:	dio40_gpio_b_out_set
*
*	Purpose:
*
*		Update the latched state of the group B GPIO pins.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		data	The data to apply.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_gpio_b_out_set(int fd, u32 data)
{
	int	err;

	err	= dio40_reg_write(fd, DIO40_GSC_COBR, data);
	return(err);
}



