// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO40/legacy/DIO40_Linux_1.00.0_GSC_DN/sbtest/clock.c $
// $Rev: 32587 $
// $Date:$

#include <stdio.h>
#include <string.h>
#include <sys/time.h>

#include "main.h"



/******************************************************************************
*
*	Function:	_a0_tx_clock_get_test
*
*	Purpose:
*
*		Verify the Port A0 Tx Clock get library services
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _a0_tx_clock_get_test(int fd)
{
	int	errs	= 0;
	u8	get;

	label("dio40_gpio_a0_tx_clock_get");

	// Clear the register and verify that the feature is off.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0);
	errs	+= dio40_gpio_a0_tx_clock_get(fd, &get);
	errs	+= get ? 1 : 0;

	// Enable the GPIO output and verify that the feature is off.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0x001);
	errs	+= dio40_gpio_a0_tx_clock_get(fd, &get);
	errs	+= get ? 1 : 0;

	// Enable the Tx Clock option ONLY and verify that the feature is off.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0x100);
	errs	+= dio40_gpio_a0_tx_clock_get(fd, &get);
	errs	+= get ? 1 : 0;

	// Enable both options and verify that the feature is on.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0x101);
	errs	+= dio40_gpio_a0_tx_clock_get(fd, &get);
	errs	+= get ? 0 : 1;

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_a0_tx_clock_set_test
*
*	Purpose:
*
*		Verify the Port A0 Tx Clock set library services
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _a0_tx_clock_set_test(int fd)
{
	u32	ccar;
	int	errs	= 0;

	label("dio40_gpio_a0_tx_clock_set");

	// Turn it on and verify that both bits are set.
	errs	+= dio40_gpio_a0_tx_clock_set(fd, 1);
	errs	+= dio40_reg_read(fd, DIO40_GSC_CCAR, &ccar);
	errs	+= (ccar & 0x100) ? 0 : 1;
	errs	+= (ccar & 0x001) ? 0 : 1;

	// Turn it off and verify that both bits are clean.
	errs	+= dio40_gpio_a0_tx_clock_set(fd, 0);
	errs	+= dio40_reg_read(fd, DIO40_GSC_CCAR, &ccar);
	errs	+= (ccar & 0x100) ? 1 : 0;
	errs	+= (ccar & 0x001) ? 1 : 0;

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_a7_tx_clock_get_test
*
*	Purpose:
*
*		Verify the Port A7 Tx Clock get library services
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _a7_tx_clock_get_test(int fd)
{
	int	errs	= 0;
	u8	get;

	label("dio40_gpio_a7_tx_clock_get");

	// Clear the register and verify that the feature is off.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0);
	errs	+= dio40_gpio_a7_tx_clock_get(fd, &get);
	errs	+= get ? 1 : 0;

	// Enable the GPIO output and verify that the feature is off.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0x00080);
	errs	+= dio40_gpio_a7_tx_clock_get(fd, &get);
	errs	+= get ? 1 : 0;

	// Enable the Tx Clock option ONLY and verify that the feature is off.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0x10000);
	errs	+= dio40_gpio_a7_tx_clock_get(fd, &get);
	errs	+= get ? 1 : 0;

	// Enable both options and verify that the feature is on.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0x10080);
	errs	+= dio40_gpio_a7_tx_clock_get(fd, &get);
	errs	+= get ? 0 : 1;

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_a7_tx_clock_set_test
*
*	Purpose:
*
*		Verify the Port A7 Tx Clock set library services
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _a7_tx_clock_set_test(int fd)
{
	u32	ccar;
	int	errs	= 0;

	label("dio40_gpio_a7_tx_clock_set");

	// Turn it on and verify that both bits are set.
	errs	+= dio40_gpio_a7_tx_clock_set(fd, 1);
	errs	+= dio40_reg_read(fd, DIO40_GSC_CCAR, &ccar);
	errs	+= (ccar & 0x10000) ? 0 : 1;
	errs	+= (ccar & 0x00080) ? 0 : 1;

	// Turn it off and verify that both bits are clean.
	errs	+= dio40_gpio_a7_tx_clock_set(fd, 0);
	errs	+= dio40_reg_read(fd, DIO40_GSC_CCAR, &ccar);
	errs	+= (ccar & 0x10000) ? 1 : 0;
	errs	+= (ccar & 0x00080) ? 1 : 0;

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	clock_test
*
*	Purpose:
*
*		Perform library Port A clock tests.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int clock_test(int fd)
{
	int	errs	= 0;

	label("Port A Clock Tests");
	printf("\n");
	label_level_inc();

	errs	+= _a0_tx_clock_get_test(fd);
	errs	+= _a0_tx_clock_set_test(fd);
	errs	+= _a7_tx_clock_get_test(fd);
	errs	+= _a7_tx_clock_set_test(fd);

	label_level_dec();
	return(errs);
}



