// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO40/legacy/DIO40_Linux_1.00.0_GSC_DN/utils/util_id.c $
// $Rev: 36252 $
// $Date:$

#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <sys/utsname.h>

#include "dio40_lib.h"
#include "dio40_dsl.h"
#include "dio40_utils.h"



/******************************************************************************
*
*	Function:	_id_board_frr
*
*	Purpose:
*
*		Identify the board from the Firmware Revision Register.
*
*	Arguments:
*
*		frr		The value read from the Firmware Revision Register.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _id_board_frr(u32 frr)
{
	int	errs	= 0;
	u32	fld;

	label("Firmware Rev. Reg.");
	printf("0x%08lX\n", (long) frr);
	label_level_inc();

	label("Firmware Revision");
	fld	= (frr >> 0) & 0xFF;
	printf("0x------%02lX\n", (long) fld);

	label("Board Version");
	fld	= (frr >> 8) & 0xFF;
	printf("0x----%02lX--\n", (long) fld);

	label("Hardware Revision");
	fld	= (frr >> 16) & 0xFF;
	printf("0x--%02lX---- ", (long) fld);

	if (fld == 0)
		printf("(no revision)\n");
	else
		printf("(Rev. %c)\n", (char) fld + 'A' - 1);

	label("Form Factor");
	fld	= (frr >> 24) & 0xF;
	printf("0x-%01lX------ ", (long) fld);

	if (fld == DIO40_GSC_FRR_FF_PCI)
		printf("(PCI)\n");
	else if (fld == DIO40_GSC_FRR_FF_PMC)
		printf("(PMC)\n");
	else if (fld == DIO40_GSC_FRR_FF_CPCI)
		printf("(cPCI)\n");
	else if (fld == DIO40_GSC_FRR_FF_PC104P)
		printf("(PC/104+)\n");
	else
		printf("(UNKNOWN)\n");

	label("Bus Size");
	fld	= (frr >> 28) & 0x1;
	printf("0x%01lX------- ", (long) fld);

	if (fld)
		printf("(64-bit)\n");
	else
		printf("(32-bit)\n");

	label("Reserved");
	fld	= (frr >> 28) & 0x5;
	printf("0x%01lX------- %s\n", (long) fld, fld ? "(UNRECOGNIZED)" : "");

	label("Feature Set Reg.");
	fld	= (frr >> 28) & 0x8;
	printf("0x%01lX------- %s\n", (long) fld, fld ? "(Yes)" : "(No)");

	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_id_board_pci
*
*	Purpose:
*
*		Identify the board from the PCI registers.
*
*	Arguments:
*
*		vidr	The PCI Vendor Id Register value.
*
*		didr	The PCI Device Id Register value.
*
*		svid	The PCI Subsystem Vendor Id Register value.
*
*		sid		The PCI Subsystem Id Register value.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _id_board_pci(u32 vidr, u32 didr, u32 svid, u32 sid)
{
	int	errs	= 0;

	label("Vendor ID");
	printf("0x%04lX     ", (long) vidr);

	if (vidr == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	label("Device ID");
	printf("0x%04lX     ", (long) didr);

	if (didr == 0x9656)
	{
		printf("(PCI9656)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	label("Sub Vendor ID");
	printf("0x%04lX     ", (long) svid);

	if (svid == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	label("Subsystem ID");
	printf("0x%04lX     ", (long) sid);

	if (sid == 0x3191)
	{
		printf("(DIO40)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_id_board_bsr
*
*	Purpose:
*
*		Identify the board from the Board Status Register.
*
*	Arguments:
*
*		bsr		The value read from the BSR.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int  _id_board_bsr(u32 bsr)
{
	u32	field;

	label("Board Status Reg.");
	printf("0x%08lX\n", (long) bsr);
	label_level_inc();

	label("User Jumpers");
	field	= (bsr >> 16) & 0x3;
	printf("0x---%01lX---- ", (long) field);
	printf("(UJ1 %s", (field & 2) ? "On" : "Off");
	printf(", UJ0 %s)\n", (field & 1) ? "On" : "Off");

	label_level_dec();
	return(0);
}



/******************************************************************************
*
*	Function:	id_board
*
*	Purpose:
*
*		Identify the board.
*
*	Arguments:
*
*		fd		The handle to use to access the board.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

int id_board(int fd)
{
	u32	bsr;
	u32	didr;
	int	errs	= 0;
	u32	frr;
	u32	sid;
	u32	svid;
	u32	vidr;

	label("Board Identification");
	errs	+= dio40_dsl_reg_read(fd, GSC_PCI_9656_VIDR, &vidr);
	errs	+= dio40_dsl_reg_read(fd, GSC_PCI_9656_DIDR, &didr);
	errs	+= dio40_dsl_reg_read(fd, GSC_PCI_9656_SVID, &svid);
	errs	+= dio40_dsl_reg_read(fd, GSC_PCI_9656_SID, &sid);
	errs	+= dio40_dsl_reg_read(fd, DIO40_GSC_FRR, &frr);
	errs	+= dio40_dsl_reg_read(fd, DIO40_GSC_BSR, &bsr);
	errs	= errs ? 1 : 0;

	if (errs == 0)
	{
		printf("\n");
		label_level_inc();
		errs	+= _id_board_pci(vidr, didr, svid, sid);
		errs	+= _id_board_frr(frr);
		errs	+= _id_board_bsr(bsr);
		label_level_dec();
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	id_library
*
*	Purpose:
*
*		Identify the Driver Interface Library.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void id_library(void)
{
	char	built[64];
	char	version[64];

	label("DIO40 Library");
	printf("\n");
	label_level_inc();

	dio40_lib_version(version, sizeof(version), built, sizeof(built));
	label("Version");
	printf("%s\n", version);
	label("Built");
	printf("%s\n", built);

	label_level_dec();
}



/******************************************************************************
*
*	Function:	id_host
*
*	Purpose:
*
*		Identify the operating system.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void id_host(void)
{
	char			buf[1024];
	int				end;
	FILE*			file;
	int				i;
	int				index;
	char*			psz;
	char			tmp[1024];
	struct utsname	uts;

	memset(&uts, 0, sizeof(uts));
	uname(&uts);
	label("Host");
	printf("\n");
	label_level_inc();

	label("Operating System");
	printf("%s\n", uts.sysname);
	label("Kernel");
	printf("%s\n", uts.release);
	label("Processor");
	printf("%s\n", uts.machine);
	label_level_inc();

	file	= fopen("/proc/cpuinfo", "r");

	if (file)
	{
		for (;;)
		{
			end	= feof(file);

			if (end)
				break;

			fgets(buf, sizeof(buf), file);
			i	= sscanf(buf, "processor : %d", &index);

			if (i == 1)
			{
				label_level_dec();
				sprintf(buf, "Processor #%d", index);
				label(buf);
				printf("\n");
				label_level_inc();
				continue;
			}

			i	= sscanf(buf, "model name : %s", tmp);

			if (i == 1)
			{
				psz	= strchr(buf, ':');

				if (psz == NULL)
					continue;

				label("CPU Model");
				psz	+= 2;

				for (; isspace(psz[0]);)
					psz++;

				printf("%s", psz);
				continue;
			}

			i	= sscanf(buf, "cpu MHz : %s", tmp);

			if (i == 1)
			{
				label("CPU Speed");
				printf("%s MHz\n", tmp);
				continue;
			}
		}

		fclose(file);
	}

	label_level_dec();
	label_level_dec();
}



