// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO40/legacy/DIO40_Linux_1.00.0_GSC_DN/utils/util_label.c $
// $Rev: 32264 $
// $Date:$

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "dio40_utils.h"



// variables ******************************************************************

static	int	_width	= 0;
static	int	_level	= 0;



/******************************************************************************
*
*	Function:	label
*
*	Purpose:
*
*		Print a message introducing a test.
*
*	Arguments:
*
*		text	The test name.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void label(const char* text)
{
	int	i;
	int	spaces;

	spaces	= _width - strlen(text);
	spaces	= (spaces < 0) ? 0 : spaces;

	for (i = 0; i < _level; i++)
	{
		printf("  ");
		spaces	-= 2;
	}

	printf("%s: %*s", text, spaces, "");
	fflush(stdout);
}



/******************************************************************************
*
*	Function:	label_init
*
*	Purpose:
*
*		Initialize the label code.
*
*	Arguments:
*
*		width	The default width for the label field.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void label_init(int width)
{
	_width	= width;
}



/******************************************************************************
*
*	Function:	label_level_dec
*
*	Purpose:
*
*		Decrease the label indenting.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void label_level_dec(void)
{
	_level--;
}



/******************************************************************************
*
*	Function:	label_level_inc
*
*	Purpose:
*
*		Increase the label indenting.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void label_level_inc(void)
{
	_level++;
}



/******************************************************************************
*
*	Function:	label_fail_func
*
*	Purpose:
*
*		Report a failure for the given function if there are errors
*		and verbose is enabled.
*
*	Arguments:
*
*		errs	Have there been any errors?
*
*		verbose	Work verbosely?
*
*		func	The name of the function that returned the error.
*
*	Returned:
*
*		1		This is an error condition.
*
*****************************************************************************/

int label_fail_func(int errs, int verbose, const char* func)
{
	if ((errs) && (verbose) && (func))
		printf("FAIL <---  (%s(), errno %d)\n", func, errno);

	return(1);
}



/******************************************************************************
*
*	Function:	label_fail_text
*
*	Purpose:
*
*		Report a failure with the given text if there are errors and
*		verbose is enabled.
*
*	Arguments:
*
*		errs	Have there been any errors?
*
*		verbose	Work verbosely?
*
*		text	The text to report.
*
*	Returned:
*
*		1		This is an error condition.
*
*****************************************************************************/

int label_fail_text(int errs, int verbose, const char* text)
{
	if ((errs) && (verbose) && (text))
		printf("FAIL <---  (%s)\n", text);

	return(1);
}



/******************************************************************************
*
*	Function:	label_fail_value
*
*	Purpose:
*
*		Report a failure with the given value if there are errors and
*		verbose is enabled.
*
*	Arguments:
*
*		errs	Have there been any errors?
*
*		verbose	Work verbosely?
*
*		value	The value to report.
*
*	Returned:
*
*		1		This is an error condition.
*
*****************************************************************************/

int label_fail_value(int errs, int verbose, long value)
{
	if ((errs) && (verbose))
	{
		printf(	"FAIL <---  (invalid value: %ld (0x%lX)\n",
				(long) value,
				(long) value);
	}

	return(1);
}



/******************************************************************************
*
*	Function:	label_format_f
*
*	Purpose:
*
*		Format a value into a string according to the given size of one K.
*		The value may be less than 1.
*
*	Arguments:
*
*		val		The value to format.
*
*		k		The size if one K. This should be 1000 or 1024.
*
*	Returned:
*
*		!NULL	A pointer to a static buffer containing the formatted value.
*
*****************************************************************************/

const char* label_format_f(float val, u32 k)
{
	static const struct
	{
		float		limit;
		float		mult;
		const char*	format;
		const char*	units;
	} small[]	=
	{
		// limit			mult			format	units
		{    0.000000001,	1000000000.,	"%.3f",	"n"		},
		{    0.00000001,	1000000000.,	"%.2f",	"n"		},
		{    0.0000001,		1000000000.,	"%.1f",	"n"		},
		{    0.000001,		1000000000.,	"%.0f",	"n"		},
		{    0.00001,		   1000000.,	"%.2f",	"u"		},
		{    0.0001,		   1000000.,	"%.1f",	"u"		},
		{    0.001,			   1000000.,	"%.0f",	"u"		},
		{    0.01,			      1000.,	"%.2f",	"m"		},
		{    0.1,			      1000.,	"%.1f",	"m"		},
		{    1.,			      1000.,	"%.0f",	"m"		},
		{   10.,			         1.,	"%.2f",	""		},
		{  100.,			         1.,	"%.1f",	""		},
		{ 1000.,				     1.,	"%.0f",	""		},
		{ 0,				         0,		NULL,	NULL	}
	};

	static const char*	large[]	= { "K", "M", "G", "T", NULL };

	static char	buf[32];
	const char*	format	= NULL;
	int			i;
	int			j;
	float		mult	= 1;
	const char*	units	= "";

	for (i = 0; small[i].units; i++)
	{
		if (val < small[i].limit)
		{
			mult	= small[i].mult;
			format	= small[i].format;
			units	= small[i].units;
			break;
		}
	}

	for (j = 0; (format == NULL) && (large[j]); j++)
	{
		val	/= k;

		for (i = 0; small[i].units; i++)
		{
			if (val < small[i].limit)
			{
				mult	= 1;
				format	= small[i].format;
				units	= large[j];
				break;
			}
		}
	}

	if (format == NULL)
	{
		mult	= 1.;
		format	= "%.0f";
		units	= "T";
	}

	val	*= mult;
	sprintf(buf, format, val);
	strcat(buf, units);
	return(buf);
}



/******************************************************************************
*
*	Function:	label_format_k
*
*	Purpose:
*
*		Format a value into a string according to the given size of one K.
*
*	Arguments:
*
*		val		The value to format.
*
*		k		The size if one K.
*
*	Returned:
*
*		!NULL	A pointer to a static buffer containing the formatted value.
*
*****************************************************************************/

const char* label_format_k(u32 val, u32 k)
{
	static char	buf[32];
	const char*	factor	= "";

	if ((val % k) == 0)
	{
		val		/= k;
		factor	= "K";
	}

	if ((val % k) == 0)
	{
		val		/= k;
		factor	= "M";
	}

	if ((val % k) == 0)
	{
		val		/= k;
		factor	= "G";
	}

	sprintf(buf, "%lu%s", (long) val, factor);
	return(buf);
}



